/*
 * Created on Sep 17, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package applet.theater;

/**
 * @author elmagk
 */

/** SALSA/World Wide Computer Project
 *  ObjectHandler recieves incoming objects
 *  and determines if they are actors or messages.
 *  upon recieving actors, it will start them up.
 *  messages get sent to the appropriate actor.
 *
 *  author Travis Desell
 */


import java.net.*;
import java.io.*;
import salsa.language.Actor;
import salsa.language.Message;
import salsa.language.RunTime;
import salsa.language.ServiceFactory;
import salsa.messaging.*;
import wwc.messaging.RMSPAcknowledgement;

public class AppletReceptionService implements ReceptionService {
	private ObjectInputStream inputStream = null;
	private ObjectOutputStream outputStream = null;

	public AppletReceptionService() {}

	public void processActor(Actor incomingState, Socket incoming) {
		//add the state to the stateRegistry, so incoming actor references can
		//get to it
		ServiceFactory.getTheater().setEntry(incomingState.getUAN(), incomingState);
		incomingState.start();
		RunTime.receivedUniversalActor();

		// Send an RmspAcknowledgement.
		try {
			outputStream.writeObject(new RMSPAcknowledgement());
			outputStream.flush(); 
		} catch (IOException e) {
			System.err.println("Reception Service error: ");
			System.err.println("\tReceiving actor: " + incomingState.toString());
			System.err.println("\tCould not send RmspAcknowledgement, is the remote theater compliant?");
			System.err.println("\tException: " + e.getMessage() );
		}
	}

	public void processMessage(Message message) {
		// Send the message.
		if (message.getTarget().getActor() == null) {
			//message.getSender()<-MessageTargetNotFound(message);
			Message NoTargetMessage = new Message(null, message.getSender(), "MessageTargetNotFound", null);
			Object[] arguments = { message };
			NoTargetMessage.setArguments( arguments );
	
			message.getSender().send(NoTargetMessage);
		} else {
			message.getTarget().send(message);
		}
	}

	public void process(Socket incoming) {
		// Read in the object.
		Object received = null;

		try {
			inputStream = new ObjectInputStream( incoming.getInputStream() );
			outputStream = new ObjectOutputStream( incoming.getOutputStream());
		} catch (IOException e) {
			System.out.println(e.getStackTrace().toString());
			System.err.println("IOException occured opening ObjectInputStream " + e);
		}

		while(true){
			try {								
				received = inputStream.readObject();
			} catch (ClassNotFoundException e) {
				System.err.println( "Could not load class for " + received.getClass().getName() );
				System.err.println( "  Is it in the theater CLASSPATH" );
				System.err.println( e.getMessage() );
				return;
			} catch (IOException e) {
				System.err.println( "IOException occured reading object " + e.getMessage() );
				return;
			}

			if (received instanceof Actor) {
				Actor incomingActor = (Actor) received;
				processActor(incomingActor, incoming);
			} else if (received instanceof Message) {  // just serializing data
				Message message = (Message) received;								
				processMessage(message);
			}
		}
	}	
}

