/*
 * 1.1 version.
 */

import javax.swing.*;
import java.net.URL;
import java.awt.*;
import java.awt.event.*;

import java.util.Vector;

import salsa.language.ServiceFactory;
import salsa.messaging.TransportService;
import salsa.messaging.ReceptionService;
import salsa.messaging.TheaterService;
import salsa.naming.NamingService;

import applet.theater.AppletTransportService;
import applet.theater.AppletNamingService;

public class AppletTheater extends JApplet implements ActionListener {
	protected JButton disconnectButton;
	protected JButton pauseButton;

	protected JList actorList;
	protected JList messagesSentList;
	protected JList messagesProcessedList;
	protected JList messagesReceivedList;
	protected JList statusList;

	protected JScrollPane actorPane;
	protected JScrollPane messagesSentPane;
	protected JScrollPane messagesProcessedPane;
	protected JScrollPane messagesReceivedPane;
	protected JScrollPane statusPane;

	protected static final String DISCONNECT = "disconnect";
	protected static final String PAUSE = "pause";

	protected String proxyURL = null;
	protected int proxyPort = 1111;

	public AppletTheater() {}

	public void init() {
		setContentPane(makeContentPane());

		Vector statusData = new Vector();
		statusList.setListData(statusData);
		statusData.add("Initializing:");

		ServiceFactory.isApplet();
		ServiceFactory.setTransportClass("applet.theater.AppletTransportService");
		ServiceFactory.setReceptionClass("applet.theater.AppletReceptionService");
		ServiceFactory.setNamingClass("applet.naming.AppletNamingService");
		ServiceFactory.setOutputClass("applet.resources.AppletoutputService");
		ServiceFactory.setErrorClass("applet.resources.AppletErrorService");
		ServiceFactory.setStorageClass("applet.resources.AppletStorageService");
		statusData.add("AppletServiceFactory configured...");

		TheaterService theater = ServiceFactory.getTheater();
		statusData.add("TheaterService (" + theater.getClass().getName() + ")... Successful.");

		AppletTransportService transport = (AppletTransportService)ServiceFactory.getTransport();
		transport.setProxyInformation(proxyURL, proxyPort);
		statusData.add("TransportService (" + transport.getClass().getName() + ")... Successful.");

		ReceptionService reception = ServiceFactory.getReception();
		statusData.add("ReceptionService (" + reception.getClass().getName() + ")... Successful.");

		AppletNamingService naming = (AppletNamingService)ServiceFactory.getNaming();
		naming.setProxyInformation(proxyURL, proxyPort);
		statusData.add("NamingService (" + naming.getClass().getName() + ")... Successful.");

		statusData.add("Finished.");
	}

	public Container makeContentPane() {
		proxyURL = getCodeBase().getHost();

		JPanel pane = new JPanel();
		pane.setLayout(null);

		//add the buttons
		disconnectButton = getDisconnectButton();
		pauseButton = getPauseButton();
		pane.add(disconnectButton);
		pane.add(pauseButton);

		//add the text labels
		JLabel actorListName = new JLabel("Actors Computing");
		JLabel messagesReceivedListName = new JLabel("Messages Received");
		JLabel messagesProcessedListName = new JLabel("Messages Processed");
		JLabel messagesSentListName = new JLabel("Messages Sent");

		JLabel welcome = new JLabel("You are now part of the World-Wide Computer!");
		JLabel connectedTo = new JLabel("You are connected to the proxy: " + proxyURL + ":" + proxyPort);

		JLabel status = new JLabel("Actions Performed");

		pane.add(actorListName);
		pane.add(messagesReceivedListName);
		pane.add(messagesProcessedListName);
		pane.add(messagesSentListName);
		pane.add(welcome);
		pane.add(connectedTo);

		//add the lists and panes

		actorList = getEmptyList();
		messagesProcessedList = getEmptyList();
		messagesReceivedList = getEmptyList();
		messagesSentList = getEmptyList();
		statusList = getEmptyList();

		actorPane = wrapListInPane(actorList);
		messagesReceivedPane = wrapListInPane(messagesReceivedList);
		messagesProcessedPane = wrapListInPane(messagesProcessedList);
		messagesSentPane = wrapListInPane(messagesSentList);
		statusPane = wrapListInPane(statusList);

		pane.add(actorPane);
		pane.add(messagesProcessedPane);
		pane.add(messagesReceivedPane);
		pane.add(messagesSentPane);
		pane.add(statusPane);

		pane.add(status);

		Insets insets = pane.getInsets();
		disconnectButton.setBounds(insets.left + 555, insets.top + 5, 240, 25);
		pauseButton.setBounds(insets.left + 555, insets.top + 30, 240, 25);

		welcome.setBounds(insets.left + 10, insets.top + 10, 550, 20);
		connectedTo.setBounds(insets.left + 10, insets.top + 35, 550, 20);

		actorListName.setBounds(insets.left + 10, insets.top + 60, 340, 20);
		messagesReceivedListName.setBounds(insets.left + 350, insets.top + 60, 150, 20);
		messagesProcessedListName.setBounds(insets.left + 500, insets.top + 60, 150, 20);
		messagesSentListName.setBounds(insets.left + 650, insets.top + 60, 150, 20);

		actorPane.setBounds(insets.left + 5, insets.top + 80, 340, 200);
		messagesReceivedPane.setBounds(insets.left + 345, insets.top + 80, 150, 200);
		messagesProcessedPane.setBounds(insets.left + 495, insets.top + 80, 150, 200);
		messagesSentPane.setBounds(insets.left + 645, insets.top + 80, 150, 200);

		status.setBounds(insets.left + 10, insets.top + 285, 300, 20);
		statusPane.setBounds(insets.left + 5, insets.top + 305, 790, 240);

		pane.setBorder(BorderFactory.createMatteBorder(1,1,2,2,Color.black));

		return pane;
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals(DISCONNECT)) {
		} else if (e.getActionCommand().equals(PAUSE)) {
			if (pauseButton.getText().equals("Pause Computation.")) {
				pauseButton.setText("Resume Computation.");
			} else {
				pauseButton.setText("Pause Computation.");
			}
		}
	}
    
	public JButton getDisconnectButton() {
		JButton disconnectButton = new JButton("Disconnect from the Computation");
		disconnectButton.setVerticalTextPosition(AbstractButton.CENTER);
		disconnectButton.setHorizontalTextPosition(AbstractButton.LEFT);
		disconnectButton.setMnemonic(KeyEvent.VK_D);
		disconnectButton.setActionCommand(DISCONNECT);
		disconnectButton.addActionListener(this);
		disconnectButton.setToolTipText("This button will disconnect the Applet Theater from IO.");

		return disconnectButton;
	}

	public JButton getPauseButton() {
		JButton pauseButton = new JButton("Pause Computation.");
		pauseButton.setVerticalTextPosition(AbstractButton.BOTTOM);
		pauseButton.setHorizontalTextPosition(AbstractButton.CENTER);
		pauseButton.setActionCommand(PAUSE);
		pauseButton.addActionListener(this);
		pauseButton.setToolTipText("This button will pause or resume all the actors running on your Applet Theater.");

		return pauseButton;
	}

	public JList getEmptyList() {
		String[] data = { };
		JList list = new JList(data); 
		list.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		list.setLayoutOrientation(JList.VERTICAL);
		list.setVisibleRowCount(5);

		return list;
	}

	public JScrollPane wrapListInPane(JList list) {
		JScrollPane listScroller = new JScrollPane(list);
		listScroller.setPreferredSize(new Dimension(250, 80));
		return listScroller;
	}

	public static void main(String[] args) {
		JFrame frame = new JFrame("Application version: AppletDemo");

		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});

		AppletTheater applet = new AppletTheater();
		frame.setContentPane(applet.makeContentPane());
		frame.pack();
		frame.setVisible(true);
	}
}

