/*
	Code generation for the following grammar segment:
	
	BehaviorBody :=
	"{" 
		(
			  Initializer 
			| NestedBehaviorDeclaration 
			| StateVariableDeclaration
			| ConstructorDeclaration
			| MethodDeclaration
		)* 
	"}"
*/

package salsac.definitions;

import salsac.SalsaCompiler;
import salsac.SalsaParser;
import salsac.SimpleNode;


public class BehaviorBody extends SimpleNode {
	
	public BehaviorBody(int id) 			{ super(id); }
	public BehaviorBody(SalsaParser p, int id)	{ super(p, id); }

	public String getPreCode()		{
		SalsaCompiler.indent++;
		String actorName = parent.getToken(1).image;

		String code = "{\n";

		/**
			Create and send a message which will invoke act on the actor.
		*/
		code += SalsaCompiler.getIndent() + "public static void main(String args[]) {\n";

		code += SalsaCompiler.getIndent() + "\tUAN uan = null;\n";
		code += SalsaCompiler.getIndent() + "\tUAL ual = null;\n";
		code += SalsaCompiler.getIndent() + "\tif (System.getProperty(\"uan\") != null) {\n";
		code += SalsaCompiler.getIndent() + "\t\tuan = new UAN( System.getProperty(\"uan\") );\n";
		code += SalsaCompiler.getIndent() + "\t\tServiceFactory.getTheater();\n";
		code += SalsaCompiler.getIndent() + "\t\tRunTime.receivedUniversalActor();\n";
		code += SalsaCompiler.getIndent() + "\t}\n";
		code += SalsaCompiler.getIndent() + "\tif (System.getProperty(\"ual\") != null) {\n";
		code += SalsaCompiler.getIndent() + "\t\tual = new UAL( System.getProperty(\"ual\") );\n\n";
		code += SalsaCompiler.getIndent() + "\t\tif (uan == null) {\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\"Actor Creation Error:\");\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\"\tuan: \" + uan);\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\"\tual: \" + ual);\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\"\tIdentifier: \" + System.getProperty(\"identifier\"));\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\"\tCannot specify an actor to have a ual at runtime without a uan.\");\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\"\tTo give an actor a specific ual at runtime, use the identifier system property.\");\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.exit(0);\n";
		code += SalsaCompiler.getIndent() + "\t\t}\n";
		code += SalsaCompiler.getIndent() + "\t\tRunTime.receivedUniversalActor();\n";
		code += SalsaCompiler.getIndent() + "\t}\n";

		code += SalsaCompiler.getIndent() + "\tif (System.getProperty(\"identifier\") != null) {\n";
		code += SalsaCompiler.getIndent() + "\t\tif (ual != null) {\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\"Actor Creation Error:\");\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\"\tuan: \" + uan);\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\"\tual: \" + ual);\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\"\tIdentifier: \" + System.getProperty(\"identifier\"));\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\"\tCannot specify an identifier and a ual with system properties when creating an actor.\");\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.exit(0);\n";
		code += SalsaCompiler.getIndent() + "\t\t}\n";
		code += SalsaCompiler.getIndent() + "\t\tual = new UAL( ServiceFactory.getTheater().getLocation() + System.getProperty(\"identifier\"));\n";
		code += SalsaCompiler.getIndent() + "\t}\n";

		code += SalsaCompiler.getIndent() + "\t" + actorName + " instance = (" + actorName + ")new " + actorName + "(uan, ual).construct();\n";

		code += SalsaCompiler.getIndent() + "\t{\n";
		code += SalsaCompiler.getIndent() + "\t\tObject[] _arguments = { args };\n";
		code += SalsaCompiler.getIndent() + "\t\tinstance.send( new Message(instance, instance, \"act\", _arguments, null, null) );\n";
		code += SalsaCompiler.getIndent() + "\t}\n";

		code += SalsaCompiler.getIndent() + "}\n\n";

		/**
			Generate code for the constructors.
			ActorName(boolean o, UAN uan) is used for getReferenceByName
			ActorName(boolean o, UAL ual) is used for get ReferenceByLocation

			ActorName(UAN uan, UAL ual) is for remote actor creation.
			ActorName(UAN uan) is for local binding.
			ActorName(UAL ual) is for specifying an actor's UAL.
			ActorName() is for anonymous actor creation.
		*/
		code += SalsaCompiler.getIndent() + "public static ActorReference getReferenceByName(UAN uan)\t{ return new " + actorName + "(false, uan); }\n";
		code += SalsaCompiler.getIndent() + "public static ActorReference getReferenceByName(String uan)\t{ return " + actorName + ".getReferenceByName(new UAN(uan)); }\n";
		code += SalsaCompiler.getIndent() + "public static ActorReference getReferenceByLocation(UAL ual)\t{ return new " + actorName + "(false, ual); }\n\n";
		code += SalsaCompiler.getIndent() + "public static ActorReference getReferenceByLocation(String ual)\t{ return " + actorName + ".getReferenceByLocation(new UAL(ual)); }\n";

		code += SalsaCompiler.getIndent() + "public " + actorName + "(boolean o, UAN __uan)\t{ super(__uan); }\n";
		code += SalsaCompiler.getIndent() + "public " + actorName + "(boolean o, UAL __ual)\t{ super(__ual); }\n\n";

		code += SalsaCompiler.getIndent() + "public " + actorName + "(UAN __uan)\t{ this(__uan, null); }\n";
		code += SalsaCompiler.getIndent() + "public " + actorName + "(UAL __ual)\t{ this(null, __ual); }\n";
		code += SalsaCompiler.getIndent() + "public " + actorName + "()\t\t{ this(null, null);  }\n";
		code += SalsaCompiler.getIndent() + "public " + actorName + "(UAN __uan, UAL __ual) {\n";
		code += SalsaCompiler.getIndent() + "\tif (__ual != null && !__ual.getLocation().equals(ServiceFactory.getTheater().getLocation())) {\n";

		String actorModule = SalsaCompiler.getCompilationUnit().getModule();
		String fullActorName = "";
		if (!actorModule.equals("")) fullActorName += actorModule + ".";
		fullActorName += SalsaCompiler.getActorName();

		code += SalsaCompiler.getIndent() + "\t\tcreateRemotely(__uan, __ual, \"" + fullActorName + "\");\n";
		code += SalsaCompiler.getIndent() + "\t} else {\n";
		code += SalsaCompiler.getIndent() + "\t\tState state = new State(__uan, __ual);\n";
		code += SalsaCompiler.getIndent() + "\t\tstate.updateSelf(this);\n";
		code += SalsaCompiler.getIndent() + "\t\tServiceFactory.getNaming().setEntry(state.getUAN(), state.getUAL(), state);\n";
		code += SalsaCompiler.getIndent() + "\t\tif (getUAN() != null) ServiceFactory.getNaming().update(state.getUAN(), state.getUAL());\n";
		code += SalsaCompiler.getIndent() + "\t}\n";
		code += SalsaCompiler.getIndent() + "}\n\n";


		boolean hasBasic = false;
		if (children != null) {
			for (int i = 0; i < children.length; i++) {
				if (getChild(i) instanceof ConstructorDeclaration) {
					if (getChild(i).getChild(1).children == null) hasBasic = true;
					code += SalsaCompiler.getIndent() + "public UniversalActor construct (" + getChild(i).getChild(1).getJavaCode() + ") {\n";
					code += SalsaCompiler.getIndent() + "\tObject[] __arguments = { " + ((FormalParameters)getChild(i).getChild(1)).getNonPrimitiveNames() + " };\n";
					code += SalsaCompiler.getIndent() + "\tthis.send( new Message(this, this, \"construct\", __arguments, null, null) );\n";
					code += SalsaCompiler.getIndent() + "\treturn this;\n";
					code += SalsaCompiler.getIndent() + "}\n\n";
				}
			}
		}
		if (!hasBasic) {
			code += SalsaCompiler.getIndent() + "public UniversalActor construct() {\n";
			code += SalsaCompiler.getIndent() + "\tObject[] __arguments = { };\n";
			code += SalsaCompiler.getIndent() + "\tthis.send( new Message(this, this, \"construct\", __arguments, null, null) );\n";
			code += SalsaCompiler.getIndent() + "\treturn this;\n";
			code += SalsaCompiler.getIndent() + "}\n\n";
		}

		/**
			Actually create the state.
			If this actor extends another actor, have it extend that actors state
		*/
		String extendsName = ((BehaviorDeclaration)parent).extendsName;
		code += SalsaCompiler.getIndent() + "public class State extends " + ((BehaviorDeclaration)parent).extendsName + ".State";
		if ( ((BehaviorDeclaration)parent).implementNames.size() > 0) {
			code += " implements ";
			for (int i = 0; i < ((BehaviorDeclaration)parent).implementNames.size(); i++) {
				code += ((BehaviorDeclaration)parent).implementNames.get(i) + ".State";
				if (i+1 != ((BehaviorDeclaration)parent).implementNames.size()) code += ", ";
			}
		}
		code += " {\n";
		SalsaCompiler.indent++;

		code += SalsaCompiler.getIndent() + "public " + actorName + " self;\n";
		code += SalsaCompiler.getIndent() + "public void updateSelf(ActorReference actorReference) {\n";
		code += SalsaCompiler.getIndent() + "\tself = (" + actorName + ")actorReference;\n";
		code += SalsaCompiler.getIndent() + "\tself.setUAN(getUAN());\n";
		code += SalsaCompiler.getIndent() + "\tself.setUAL(getUAL());\n";
		code += SalsaCompiler.getIndent() + "}\n\n";

		code += SalsaCompiler.getIndent() + "public State() {\n";
		code += SalsaCompiler.getIndent() + "\tthis(null, null);\n";
		code += SalsaCompiler.getIndent() + "}\n\n";

		code += SalsaCompiler.getIndent() + "public State(UAN __uan, UAL __ual) {\n";
		code += SalsaCompiler.getIndent() + "\tsuper(__uan, __ual);\n";
 		code += SalsaCompiler.getIndent() + "\taddClassName( \"";
		String module = SalsaCompiler.getCompilationUnit().getModule();
		if (!module.equals("")) code += module + ".";
		code += SalsaCompiler.getActorName() + "$State\" );\n";

		code += SalsaCompiler.getIndent() + "\taddMethodsForClasses();\n";
		code += SalsaCompiler.getIndent() + "}\n\n";

		if (!hasBasic) {
			code += SalsaCompiler.getIndent() + "public void construct() {}\n\n";
		}

		code += SalsaCompiler.getIndent() + "public void process(Message message) {\n";
		code += SalsaCompiler.getIndent() + "\tMethod[] matches = getMatches(message.getMethodName());\n";
		code += SalsaCompiler.getIndent() + "\tObject returnValue = null;\n";
		code += SalsaCompiler.getIndent() + "\tException exception = null;\n\n";

		code += SalsaCompiler.getIndent() + "\tif (matches != null) {\n";
		code += SalsaCompiler.getIndent() + "\t\tfor (int i = 0; i < matches.length; i++) {\n";
		code += SalsaCompiler.getIndent() + "\t\t\ttry {\n";

		code += SalsaCompiler.getIndent() + "\t\t\t\tif (matches[i].getParameterTypes().length != message.getArguments().length) continue;\n";

		code += SalsaCompiler.getIndent() + "\t\t\t\treturnValue = matches[i].invoke(this, message.getArguments());\n";
		code += SalsaCompiler.getIndent() + "\t\t\t} catch (Exception e) {\n";
		code += SalsaCompiler.getIndent() + "\t\t\t\tif (e.getCause() instanceof CurrentContinuationException) {\n";
		code += SalsaCompiler.getIndent() + "\t\t\t\t\tsendGeneratedMessages();\n";
		code += SalsaCompiler.getIndent() + "\t\t\t\t\treturn;\n";
		code += SalsaCompiler.getIndent() + "\t\t\t\t} else if (e instanceof InvocationTargetException) {\n";
		code += SalsaCompiler.getIndent() + "\t\t\t\t\tsendGeneratedMessages();\n";
		code += SalsaCompiler.getIndent() + "\t\t\t\t\texception = (Exception)e.getCause();\n";
		code += SalsaCompiler.getIndent() + "\t\t\t\t\tbreak;\n";
		code += SalsaCompiler.getIndent() + "\t\t\t\t} else {\n";
		code += SalsaCompiler.getIndent() + "\t\t\t\t\tcontinue;\n";
		code += SalsaCompiler.getIndent() + "\t\t\t\t}\n";

		code += SalsaCompiler.getIndent() + "\t\t\t}\n";
		code += SalsaCompiler.getIndent() + "\t\t\tsendGeneratedMessages();\n";
		code += SalsaCompiler.getIndent() + "\t\t\tcurrentMessage.resolveContinuations(returnValue);\n";
		code += SalsaCompiler.getIndent() + "\t\t\treturn;\n";
		code += SalsaCompiler.getIndent() + "\t\t}\n";
		code += SalsaCompiler.getIndent() + "\t}\n\n";


		code += SalsaCompiler.getIndent() + "\tSystem.err.println(\"Message processing exception:\");\n";
		code += SalsaCompiler.getIndent() + "\tif (message.getSource() != null) {\n";
		code += SalsaCompiler.getIndent() + "\t\tSystem.err.println(\"\\tSent by: \" + message.getSource().toString());\n";
		code += SalsaCompiler.getIndent() + "\t} else System.err.println(\"\\tSent by: unknown\");\n";
		code += SalsaCompiler.getIndent() + "\tSystem.err.println(\"\\tReceived by actor: \" + toString());\n";
		code += SalsaCompiler.getIndent() + "\tSystem.err.println(\"\\tMessage: \" + message.toString());\n";
		code += SalsaCompiler.getIndent() + "\tif (exception == null) {\n";
		code += SalsaCompiler.getIndent() + "\t\tif (matches == null) {\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\"\\tNo methods with the same name found.\");\n";
		code += SalsaCompiler.getIndent() + "\t\t\treturn;\n";
		code += SalsaCompiler.getIndent() + "\t\t}\n";
		code += SalsaCompiler.getIndent() + "\t\tSystem.err.println(\"\\tDid not match any of the following: \");\n";
		code += SalsaCompiler.getIndent() + "\t\tfor (int i = 0; i < matches.length; i++) {\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.print(\"\\t\\tMethod: \" + matches[i].getName() + \"( \");\n";
		code += SalsaCompiler.getIndent() + "\t\t\tClass[] parTypes = matches[i].getParameterTypes();\n";
		code += SalsaCompiler.getIndent() + "\t\t\tfor (int j = 0; j < parTypes.length; j++) {\n";
		code += SalsaCompiler.getIndent() + "\t\t\t\tSystem.err.print(parTypes[j].getName() + \" \");\n";
		code += SalsaCompiler.getIndent() + "\t\t\t}\n";
		code += SalsaCompiler.getIndent() + "\t\t\tSystem.err.println(\")\");\n";
		code += SalsaCompiler.getIndent() + "\t\t}\n";
		code += SalsaCompiler.getIndent() + "\t} else {\n";
		code += SalsaCompiler.getIndent() + "\t\tSystem.err.println(\"\\tThrew exception: \" + exception);\n";
		code += SalsaCompiler.getIndent() + "\t\texception.printStackTrace();\n";
		code += SalsaCompiler.getIndent() + "\t}\n";
		code += SalsaCompiler.getIndent() + "}\n\n";

		return code;
	}

	public String getPostCode() {
		SalsaCompiler.indent--;

		String code = SalsaCompiler.getIndent() + "}\n";
		code += "}";
	
		return code;
	}

	public String getChildCode() {
		String code = "";

		if (children != null) {		
			for (int i = 0; i < children.length; i++) {
				code += SalsaCompiler.getIndent() + getChild(i).getJavaCode();
			}
		}
		
		return code;
	}
}
