/*
 * Created on Sep 17, 2003
 *
 */

package wwc.messaging;

import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Vector;
import java.util.Hashtable;

import salsa.language.*;
import salsa.language.exceptions.*;
import salsa.naming.*;
import salsa.messaging.*;


/**
 * This class is the reference implementation of the Persistent MessagingService. The
 * service is implemented as a RMSP client.
 * 
 * This implementation uses the NamingService interface and the salsa.language.
 * ServiceFactory for UAN resolution.
 * 
 * @authors Travis Desell, Kaoutar El Maghraoui
 */
 
public class WWCTransportService implements TransportService {
	/**
	 * Constructor for Messenger.
	 */
	private int maxSockets = 5;
	private int currentSocket = 0;
	private OutgoingSocketHandler[] socketPool;

	public WWCTransportService() {
		String sockets = System.getProperty("sockets");
		if (sockets != null) maxSockets = new Integer(sockets).intValue();

		socketPool = new OutgoingSocketHandler[maxSockets];		
		for (int i = 0; i < maxSockets; i++) socketPool[i] = new OutgoingSocketHandler(i);
	}
	
	public void migrate(Actor state, UAL target){
		socketPool[currentSocket].put(state, target);

		currentSocket++;
		if (currentSocket >= maxSockets) currentSocket = 0;
	}

	public void send(Message message, ActorReference target) {
		socketPool[currentSocket].put(message, target.getUAL());

		currentSocket = (currentSocket++)%maxSockets;
	}
}



